package applets;

import shout3d.*;
import shout3d.core.*;
import shout3d.math.*;


public class RotatePanel extends Shout3DPanel implements DeviceObserver {
	
	//Node in scene to rotate
	Transform boxTrans;

	//pixels on screen
	int pixelStartX;
	int pixelStartY;
	int pixelEndX;
	int pixelEndY;


	//to store and compute angles
	float [] eulers = new float [3];
	float [] axisAngle = new float [4];
	Quaternion q = new Quaternion();
	

	public RotatePanel(Shout3DApplet applet){
		super(applet);
	}
		
    	public void customInitialize() {
        
		addDeviceObserver(this,"MouseInput", null);
		boxTrans = (Transform) getNodeByName("Box01");
		
		//get starting rotation value from scene
		axisAngle = boxTrans.rotation.getValue();
		
		//put this rotation into the quaternion
		q.setAxisAngle(axisAngle);
		
		//get it back out as eulers
		q.getEulers(eulers);
		
	
	}


	protected void finalize()  { 
		removeDeviceObserver(this,"MouseInput");
	}

	
	
	public boolean onDeviceInput(DeviceInput di, Object userData) {
		MouseInput mi = (MouseInput) di;
		switch (mi.which){

			case MouseInput.DOWN:
				pixelStartX = mi.x;
				pixelStartY = mi.y;
				return true;

			case MouseInput.DRAG:

				pixelEndX = mi.x;
				pixelEndY = mi.y;

				int dragDistanceX = pixelEndX - pixelStartX;
				int dragDistanceY = pixelEndY - pixelStartY;
				
				//convert drag to rotation
				//at 50 pixels per radian (57.2 degrees)
				float headingDelta = dragDistanceX/50f;
				float pitchDelta = dragDistanceY/50f;
				
				//compute new heading and pitch
				eulers[0] = eulers[0] + headingDelta;
				eulers[1] = eulers[1] + pitchDelta;

				//Convert from Eulers to AxisAngle
				// using the Quaternion
				q.setEulers(eulers);
				q.getAxisAngle(axisAngle);

				//set the Transform to
				//updated axis angle values
				boxTrans.rotation.setValue(axisAngle);

				pixelStartX = pixelEndX;
				pixelStartY = pixelEndY;
							
				return true;

		}//end of switch

		return false;

	}


}//end of class	